#!/usr/bin/env python
# 
# Usage:
#    from calc_alpha_RJ import ( calc_alphaRJmol_following_dzliu, 
#                                calc_alphaRJtot_following_dzliu, 
#                                calc_alphaRJmol_Hughes2017, 
#                                calc_alphaRJmol_Hughes2017 )
# 

from __future__ import print_function

import os, sys, re, json, time, astropy
import numpy as np
from astropy.table import Table, Column, hstack
from copy import copy

if not (os.path.dirname(os.path.abspath(__file__)) in sys.path): sys.path.append(os.path.dirname(os.path.abspath(__file__)))
import apply_cosmology
cosmo = apply_cosmology.cosmo

if sys.version_info.major >= 3:
    long = int
else:
    pass



# 
# def 
# 










#!/usr/bin/env python
# 

from __future__ import print_function

import os, sys, re, json, time, astropy
import numpy as np
from astropy.table import Table, Column, hstack
from copy import copy
from numpy import log10, power as pow

from astropy.cosmology import FlatLambdaCDM
cosmo = FlatLambdaCDM(H0=73, Om0=0.27, Tcmb0=2.725)

if sys.version_info.major >= 3:
    long = int
else:
    pass



# 
# def 
# 
def calc_cosmic_star_formation_rate_density_MadauDickinson2014(z):
    # Madau & Dickinson (2014)
    # converted to Chabrier IMF from Salpeter IMF
    rho_SFR = 0.015 * (1+z)**2.7 / (1.0 + ((1+z)/2.9)**5.6) / 1.64
    return rho_SFR











#!/usr/bin/env python
# 

from __future__ import print_function

import os, sys, re, json, time, astropy
import numpy as np
from astropy.table import Table, Column, hstack
from copy import copy
from numpy import log10, power as pow

from astropy.cosmology import FlatLambdaCDM
cosmo = FlatLambdaCDM(H0=73, Om0=0.27, Tcmb0=2.725)

if sys.version_info.major >= 3:
    long = int
else:
    pass



# 
# def 
# 
def calc_Sargent2014_sSFR(z, lgMstar=10.5, deltaMS=0.0):
    return 0.095 * 10**(-0.21*(lgMstar-np.log10(5e10))) * np.exp(2.05*z/(1.0+0.16*z**1.54)) * 10**(deltaMS)
def calc_Speagle2014_sSFR(cosmoAge, lgMstar=10.5, deltaMS=0.0):
    return 10**((0.84 - 0.026*cosmoAge) * lgMstar - (6.51 - 0.11*cosmoAge)) / 10**(lgMstar) * 1e9 * 10**(deltaMS)
def calc_Scoville2017_sSFR(z, lgMstar=10.5, deltaMS=0.0):
    lgMstar_ref = 10.5
    SFR_MS_ref = 10**(0.59*lgMstar_ref-5.77)* np.power((1.0+z),(0.22*lgMstar_ref+0.59))
    SFR_MS = SFR_MS_ref * 10**(1.72-np.log10(1+np.power(10**(lgMstar-10.31),-1.07))) / 10**(1.72-np.log10(1+np.power(10**(lgMstar_ref-10.31),-1.07)))
    sSFR_MS = SFR_MS / 10**(lgMstar) * 1e9 # Gyr
    return sSFR_MS
def calc_sSFR_MS(lgMstar, z, cosmoAge=None):
    if cosmoAge is None:
        cosmoAge = cosmo.age(z).value
    sSFR_MS = calc_Speagle2014_sSFR(cosmoAge, lgMstar)
    # 
    #sSFR_MS = calc_Sargent2014_sSFR(z, lgMstar)
    # 
    return sSFR_MS



def calc_SFR_MS_Scoville2017(z, lgMstar=10.5):
    lgMstar_ref = 10.5
    SFR_MS_ref = 10**(0.59*lgMstar_ref-5.77)* np.power((1.0+z),(0.22*lgMstar_ref+0.59))
    SFR_MS = SFR_MS_ref * 10**(1.72-np.log10(1+np.power(10**(lgMstar-10.31),-1.07))) / 10**(1.72-np.log10(1+np.power(10**(lgMstar_ref-10.31),-1.07)))
    sSFR_MS = SFR_MS / 10**(lgMstar) * 1e9 # Gyr
    return SFR_MS


def calc_SFR_MS_Tacconi2018(z, lgMstar=10.5, with_dzliu_t_z_conversion = False):
    # They used Speagle2014 time evolution equation, but applied a cosmic time to redshift conversion of:
    # t_cosmic_age = pow(10.0, 1.143 - 1.026 * log10(1+z) - 0.599 * (log10(1+z))**2 + 0.528 * (log10(1+z))**3)
    t_cosmic_age = pow(10.0, 1.143 - 1.026 * log10(1+z) - 0.599 * (log10(1+z))**2 + 0.528 * (log10(1+z))**3)
    if with_dzliu_t_z_conversion:
        # only valid for cosmo = FlatLambdaCDM(H0=73, Om0=0.27, Tcmb0=2.725)
        x = log10(1+z)
        y = (1.1295867233) + (-1.0876769081) * x + (-0.5392282897) * x**2 + (0.2310061623) * x**3
        t_cosmic_age = pow(10.0, y)
    # 
    sSFR_MS = calc_Speagle2014_sSFR(t_cosmic_age, lgMstar)
    SFR_MS = sSFR_MS / 1e9 * 10**(lgMstar)
    return SFR_MS


def calc_SFR_MS_Speagle2014(z, lgMstar=10.5, cosmic_age=None):
    # Speagle 2014 Table 7 #49, same as calc_SFR_MS_Speagle2014_with_t()
    if cosmic_age is None:
        cosmoAge = cosmo.age(z).value
    else:
        cosmoAge = cosmic_age
    # 
    SFR_MS = 10**((0.84 - 0.026*cosmoAge) * lgMstar - (6.51 - 0.11*cosmoAge))
    sSFR_MS = SFR_MS / 10**(lgMstar) * 1e9 # Gyr
    return SFR_MS


def calc_SFR_MS_Speagle2014_with_t(z, lgMstar=10.5, cosmic_age=None):
    # Speagle 2014 Table 7 #49
    if cosmic_age is None:
        cosmoAge = cosmo.age(z).value
    else:
        cosmoAge = cosmic_age
    # 
    cosmoAge = cosmo.age(z).value
    SFR_MS = 10**((0.84 - 0.026*cosmoAge) * lgMstar - (6.51 - 0.11*cosmoAge))
    sSFR_MS = SFR_MS / 10**(lgMstar) * 1e9 # Gyr
    return SFR_MS


def calc_SFR_MS_Speagle2014_with_z(z, lgMstar=10.5, cosmic_age=None):
    # Speagle 2014 Table 8 #49
    # -- used by Scoville+2016,2017
    if cosmic_age is None:
        cosmoAge = cosmo.age(z).value
    else:
        cosmoAge = cosmic_age
    # 
    SFR_MS = 10**((0.22 * lgMstar + 0.59) * np.log10(1.+z) + 0.59 * lgMstar - 5.77)
    sSFR_MS = SFR_MS / 10**(lgMstar) * 1e9 # Gyr
    return SFR_MS


def calc_SFR_MS_Sargent2014(z, lgMstar=10.5):
    # Sargent,2014ApJ...793...19S
    # Appendix, Eq.(A1)
    #   sSFR = N(M*) * exp(A*z / (1+B*z^C))
    #   where N(M⋆) = N(5 × 10^{10} M⊙) 10^{ν log(M⋆/[5×10^10 M⊙])}
    #     N(5 × 10^{10} M⊙) = 0.095^{+0.002}_{-0.003} Gyr^{-1}
    #     ν is the slope of the log(sSFR) versus log(M⋆) relation, ν = -0.21±0.04
    #     A = 2.05+0.33−0.20
    #     B = 0.16+0.15−0.07
    #     C = 1.54±0.32
    # 
    sSFR_MS = 0.095 * 10**(-0.21*(lgMstar-np.log10(5e10))) * np.exp(2.05*z/(1.0+0.16*z**1.54)) # Gyr
    SFR_MS = sSFR_MS * 10**(lgMstar) / 1e9
    return SFR_MS


def calc_SFR_MS_Bethermin2015(z, lgMstar=10.5):
    # see paper http://arxiv.org/pdf/1409.5796v2
    #     section 4.2
    #     paragraph 1 
    #     sSFR = (0.061±0.006Gyr**{−1}) * (1+z)**{2.82±0.12} at z<2 and as (1+z)**{2.2±0.3} at z>2.
    mask = (z>=2.0)
    Bethermin2014_sSFR = (0.061 * (1.+z)**2.82)
    Bethermin2014_sSFR[mask] = ((1.+z[mask])**2.2)  / (1.+2)**2.2 * (0.061 * (1.+2)**2.82) # renormalize to z=2
    Bethermin2014_SFR = (Bethermin2014_sSFR / 1e9 * 10**(lgMstar))
    SFR_MS = Bethermin2014_SFR
    sSFR_MS = Bethermin2014_sSFR
    return SFR_MS


def calc_SFR_MS_Schreiber2015(z, lgMstar=10.5):
    # Schreiber 2015 - 2015A&A...575A..74S.pdf
    # see their Eq.(9)
    #   log10(SFR_MS[M⊙/yr]) = m − m0 + a0*r - a1 * max([0, m - m1 - a2*r]) ^ 2
    #   where r = log10(1 + z), m = log10(M∗/1e9 M⊙), and m0,m1,a0,a1,a2 are constants. 
    # Here we converts Chabrier IMF values to Salpeter IMF values for internal calculation
    Schreiber2014_lgMstar = np.array(lgMstar)+np.log10(1.73) # converts Chabrier IMF values to Salpeter IMF values for internal calculation
    Schreiber2014_SFR = Schreiber2014_lgMstar-9.0-0.50+1.5*np.log10(1.+z)
    Schreiber2014_mmr = Schreiber2014_lgMstar-9.0-0.36-2.5*np.log10(1.+z)
    Schreiber2014_mask = (Schreiber2014_mmr>0)
    Schreiber2014_SFR[Schreiber2014_mask] = (Schreiber2014_SFR[Schreiber2014_mask]-0.30*Schreiber2014_mmr[Schreiber2014_mask]**2)
    Schreiber2014_SFR = np.power(10.0, Schreiber2014_SFR-np.log10(1.73)) # converts back to Chabrier IMF values
    SFR_MS = Schreiber2014_SFR
    sSFR_MS = SFR_MS / 10**(lgMstar) * 1e9
    return SFR_MS


def calc_SFR_MS_Whitaker2014(z, lgMstar=10.5, use_equation_3 = False):
    # Whitaker,2014ApJ...795..104W
    # star forming main-sequence down to lower stellar mass at z<3
    # The evolution of sSFR, see their Eq.(6) on page 11, 
    #   sSFR/[Gyr^{-1}] = a * (1+z)^b
    #   where a and b are factors scaled with stellar mass. Using UV+IR SFR they obtain a b parameters as listed in Table 4. 
    # But at each redshift, alternatively see their Eq.(3) on page 7
    #   logSFR = a(z) (log(M*)-10.2) + b(z)
    # 
    # -- used by Genzel+2015 and Tacconi+2018
    # 
    spl_lgMstar_1 = np.array([9.2, 9.4, 9.6, 9.8, 10.0, 10.2, 10.4, 10.6, 10.8, 11.0 ])
    spl_lgMstar_2 = np.array([9.4, 9.6, 9.8, 10.0, 10.2, 10.4, 10.6, 10.8, 11.0, 11.2 ])
    spl_lga = np.array([-9.54, -9.50, -9.54, -9.58, -9.69, -9.93 , -10.11, -10.28, -10.53, -10.65 ])
    spl_b = np.array([1.95, 1.86, 1.90, 1.98, 2.16, 2.63, 2.88, 3.03, 3.37, 3.45 ])
    lga = np.interp(lgMstar, (spl_lgMstar_1+spl_lgMstar_2)/2.0, spl_lga, left=np.nan, right=np.nan)
    b = np.interp(lgMstar, (spl_lgMstar_1+spl_lgMstar_2)/2.0, spl_b, left=np.nan, right=np.nan)
    a = np.power(10.0, lga)
    sSFR_MS = (a * (1.+z)**b) # Gyr
    SFR_MS = sSFR_MS * np.power(10.0, lgMstar)
    # 
    if use_equation_3:
        spl_z_1 = np.array([0.5, 1.0, 1.5, 2.0 ])
        spl_z_2 = np.array([1.0, 1.5, 2.0, 2.5 ])
        spl_a_low = np.array([0.94, 0.99, 1.04, 0.91 ])
        spl_a_high = np.array([0.14, 0.51, 0.62, 0.67 ])
        spl_b = np.array([1.11, 1.31, 1.49, 1.62 ])
        a_low = np.interp(z, (spl_z_1+spl_z_2)/2.0, spl_a_low, left=np.nan, right=np.nan)
        a_high = np.interp(z, (spl_z_1+spl_z_2)/2.0, spl_a_high, left=np.nan, right=np.nan)
        b = np.interp(z, (spl_z_1+spl_z_2)/2.0, spl_b, left=np.nan, right=np.nan)
        SFR_MS_low = np.power(10.0, a_low * (lgMstar-10.2) + b)
        SFR_MS_high = np.power(10.0, a_high * (lgMstar-10.2) + b)
        SFR_MS = SFR_MS_low
        SFR_MS[(lgMstar>10.2)] = SFR_MS_high[(lgMstar>10.2)]
    # 
    return SFR_MS


def calc_SFR_MS_Lee2015(z, lgMstar=10.5):
    # Tomczak,2015ApJ...801...80L
    # star forming main-sequence with Herschel
    # see their Eq(2) on page 6
    # -- their equation can be rewritten as: SFR ~ 10^{s0} / ( 1 + (Mstar/M0)^{-gamma} )
    # -- their equation is also used in Tomczak2016
    spl_z_1 = np.array([0.25, 0.46, 0.63, 0.78, 0.93, 1.11])
    spl_z_2 = np.array([0.46, 0.63, 0.78, 0.93, 1.11, 1.30])
    spl_s0 = np.array([0.80, 0.99, 1.23, 1.35, 1.53, 1.72])
    spl_lgM0 = np.array([10.03, 9.82, 9.93, 9.96, 10.10, 10.31])
    spl_gamma = np.array([0.92, 1.13, 1.11, 1.28, 1.26, 1.07])
    lgM0 = np.interp(z, (spl_z_1+spl_z_2)/2.0, spl_lgM0, left=np.nan, right=np.nan)
    s0 = np.interp(z, (spl_z_1+spl_z_2)/2.0, spl_s0, left=np.nan, right=np.nan)
    gamma = np.interp(z, (spl_z_1+spl_z_2)/2.0, spl_gamma, left=np.nan, right=np.nan)
    log_SFR = s0 - np.log10(1.0 + np.power(np.power(10.0, lgMstar-lgM0), -gamma))
    SFR_MS = np.power(10.0, log_SFR)
    return SFR_MS


def calc_SFR_MS_Tomczak2016(z, lgMstar=10.5):
    # Tomczak,2016ApJ...817..118T
    # star forming main-sequence with ZFOURGE Survey
    # see their Eq(2) on page 9, and for star-forming galaxies Eq(4)
    # -- their equation can be rewritten as: SFR ~ 10^{s0} / ( 1 + (Mstar/M0)^{-gamma} )
    lgM0 = 9.458 + 0.865 * z - 0.132 * z**2
    s0 = 0.448 + 1.220 * z - 0.174 * z**2
    gamma = 1.091
    log_SFR = s0 - np.log10(1.0 + np.power(np.power(10.0, lgMstar-lgM0), -gamma))
    SFR_MS = np.power(10.0, log_SFR)
    return SFR_MS


def calc_SFR_MS_Pearson2018(z, lgMstar=10.5):
    # Pearson 2018 A&A
    # Herschel deblended IR SFR
    # abstract: log(SFR) = α[log(M⋆) − 10.5] + β
    #   β(z)=(1.10±0.07)+[(0.53±0.05)×log({0.03±0.11}+z)], Eq(9)
    #   α(z) = (0.38 ± 0.04) + (0.12 ± 0.02)z, Eq(10)
    Pearson2018_alpha = 0.38 + 0.12 * z
    Pearson2018_beta = 1.10 + (0.53 * np.log(0.03 + z)) # np.log is ln()
    SFR_MS = np.power(10.0, Pearson2018_alpha * (lgMstar-10.5) + Pearson2018_beta)
    sSFR_MS = SFR_MS / 10**(lgMstar) * 1e9 # Gyr
    return SFR_MS

def calc_SFR_MS_Leslie20180901(z, lgMstar, params = (0.5701,9.75469,4.1049,2.83077) ):
    # priv. comm.
    r = np.log10(1.+z)
    s0, m0, a1, a2 = params   
    S0_ = s0 + a1*r 
    M0_ = m0 + a2*r 
    gamma = 1.0
    log_SFR = S0_ - np.log10(1+10**(-gamma*(lgMstar-M0_)))
    SFR_MS = np.power(10.0, log_SFR)
    return SFR_MS

def calc_SFR_MS_Leslie20190111(z, lgMstar, params = (0.568,9.684,4.07,2.928,11.389) ):
    # priv. comm.
    # using a Chabrier IMF and a flat lambda CDM cosmology with H0=70,Om0=0.3.
    r = np.log10(1.+z)
    s0, m0, a1, a2, a3 = params   
    S0_ = s0 + a1*r 
    M0_ = m0 + a2*r 
    gamma = 1+np.exp(-a3*r)
    log_SFR = S0_ - np.log10(1+10**(-gamma*(lgMstar-M0_)))
    SFR_MS = np.power(10.0, log_SFR)
    return SFR_MS











#!/usr/bin/env python
# 
# usage:
#     from calc_galaxy_size_evolution import (calc_galaxy_size_Tacconi2018,
#                                             )
# 

from __future__ import print_function

import os, sys, re, json, time, astropy
import numpy as np
from astropy.table import Table, Column, hstack
from copy import copy
from numpy import log10, power as pow

if not (os.path.dirname(os.path.abspath(__file__)) in sys.path): sys.path.append(os.path.dirname(os.path.abspath(__file__)))
import apply_cosmology
cosmo = apply_cosmology.cosmo

if sys.version_info.major >= 3:
    long = int
else:
    pass



# 
# def 
# 
def calc_galaxy_size_Tacconi2018(z, Mstar):
    # Tacconi2018 Sect. 4.1
    #Mstar = 10**np.array(lgMstar)
    Re0 = 8.9 * (1.+z)**(-0.75) * (Mstar/5e10)**(0.23) # kpc
    return Re0










#!/usr/bin/env python
# 

from __future__ import print_function

import os, sys, re, json, time, astropy
import numpy as np
from astropy.table import Table, Column, hstack
from copy import copy
from numpy import log, log10, power, sum, sqrt, pi, exp
pow = power
lg = log10
ln = log
from scipy.interpolate import InterpolatedUnivariateSpline, interp1d

from astropy.cosmology import FlatLambdaCDM
cosmo = FlatLambdaCDM(H0=73, Om0=0.27, Tcmb0=2.725)

if sys.version_info.major >= 3:
    long = int
else:
    pass




# 
# def 
# 
def Schechter_Function(lgM, phi, lg_M0, alpha):
    # 
    # Schechter (1976)
    # 
    # Phi(M) dM = (Phi_*) * (M/M_*)**(alpha) * exp(-M/M_*) dM/M_*
    #           = (Phi_*) * x**(alpha) * exp(-x) dx
    #           = (Phi_*) * 10**(lgx * alpha) * exp(-10**lgx) dx
    #           = (Phi_*) * 10**(lgx * alpha) * exp(-10**lgx) dlnx
    #           = (Phi_*) * 10**(lgx * alpha) * exp(-10**lgx) dlgx * ln(10)
    #           = ln(10) * (Phi_*) * 10**((lgM-lgM_*)*(alpha+1)) * exp(-10**(lgM-lgM_*))
    # 
    lgx = lgM-lg_M0
    Phi_Schechter = phi * (10**(lgx*(alpha+1))) * (np.exp(-10**lgx))*ln(10)
    return Phi_Schechter




# 
# def 
# 
def calc_SMF_Davidzon2017(z, lgMstar=None, galaxy_type = 'SFG'):
    # 
    # Davidzon 2017 arXiv:1701.02734
    # IMF: Chabrier 2003
    # Outputs: lgMstar_grid, lgPhiMstar_grid
    # 
    # check z
    if not np.isscalar(z):
        raise ValueError('Please input a float number as the redshift!')
    # 
    # check galaxy_type
    if not (type(galaxy_type) is str):
        raise ValueError('Please input either "ALL", "SFG" or "QG" as the galaxy_type!')
    else:
        if not (galaxy_type in ['ALL', 'SFG', 'QG']):
            raise ValueError('Please input either "ALL", "SFG" or "QG" as the galaxy_type!')
    # 
    # make lgMstar
    if lgMstar is None:
        lgMstar_grid = np.linspace(6.0, 13.0, num=1000, endpoint=True)
    else:
        lgMstar_grid = lgMstar
    # 
    # read SMF
    tb_SMF = Table.read(os.path.dirname(os.path.dirname(__file__))+os.sep+'Data_Tables/datatables_SMF/datatable_Davidzon2017_SMF_'+galaxy_type+'.txt', format='ascii') # zLo zHi lgMchar  Phi_1   alpha_1  Phi_2  alpha_2
    SMF_zmin = np.min(tb_SMF['zLo'])
    SMF_zmax = np.max(tb_SMF['zHi'])
    # 
    # check z
    if z < SMF_zmin or z > SMF_zmax:
        raise ValueError('calc_SMF_Davidzon2017: The input redshift is out of the allowed range of %s -- %s!'%(SMF_zmin, SMF_zmax))
    # 
    # spline SMF #<20190214># old method
    #<20190214># SMF_z = (tb_SMF['zLo'].data + tb_SMF['zHi'].data) / 2.0
    #<20190214># SMF_phi_1 = InterpolatedUnivariateSpline(SMF_z, tb_SMF['Phi_1'].data, k=1)(z)
    #<20190214># SMF_phi_2 = InterpolatedUnivariateSpline(SMF_z, tb_SMF['Phi_2'].data, k=1)(z)
    #<20190214># SMF_alpha_1 = InterpolatedUnivariateSpline(SMF_z, tb_SMF['alpha_1'].data, k=1)(z)
    #<20190214># SMF_alpha_2 = InterpolatedUnivariateSpline(SMF_z, tb_SMF['alpha_2'].data, k=1)(z)
    #<20190214># SMF_lgMchar = InterpolatedUnivariateSpline(SMF_z, tb_SMF['lgMchar'].data, k=1)(z)
    #<20190214># #print('z, lgMchar, alpha_1, phi_1, alpha_2, phi_2 =', z, SMF_lgMchar, SMF_alpha_1, SMF_phi_1, SMF_alpha_2, SMF_phi_2)
    #<20190214># SMF_PhiMstar = Schechter_Function(lgMstar_grid, SMF_phi_1, SMF_lgMchar, SMF_alpha_1) + \
    #<20190214>#                Schechter_Function(lgMstar_grid, SMF_phi_2, SMF_lgMchar, SMF_alpha_2) # two component
    #<20190214># lgPhiMstar_grid = np.log10(SMF_PhiMstar)
    # 
    # spline SMF
    lgPhiMstar_matrix = []
    for k in range(len(tb_SMF)):
        SMF_z = (tb_SMF['zLo'][k] + tb_SMF['zHi'][k]) / 2.0
        SMF_phi_1 = tb_SMF['Phi_1'][k]
        SMF_phi_2 = tb_SMF['Phi_2'][k]
        SMF_alpha_1 = tb_SMF['alpha_1'][k]
        SMF_alpha_2 = tb_SMF['alpha_2'][k]
        SMF_lgMchar = tb_SMF['lgMchar'][k]
        #print('z, lgMchar, alpha_1, phi_1, alpha_2, phi_2 =', z, SMF_lgMchar, SMF_alpha_1, SMF_phi_1, SMF_alpha_2, SMF_phi_2)
        SMF_PhiMstar = Schechter_Function(lgMstar_grid, SMF_phi_1, SMF_lgMchar, SMF_alpha_1) + \
                       Schechter_Function(lgMstar_grid, SMF_phi_2, SMF_lgMchar, SMF_alpha_2) # two component
        lgPhiMstar_grid = np.log10(SMF_PhiMstar)
        lgPhiMstar_matrix.append(copy(lgPhiMstar_grid))
    # 
    SMF_z = (tb_SMF['zLo'].data + tb_SMF['zHi'].data) / 2.0
    lgPhiMstar_matrix = np.array(lgPhiMstar_matrix) # shape == (N_SMF_z, N_SMF_lgMstar, )
    if z <= np.min(SMF_z):
        lgPhiMstar_grid = lgPhiMstar_matrix[0]
    elif z >= np.max(SMF_z):
        lgPhiMstar_grid = lgPhiMstar_matrix[-1]
    else:
        lgPhiMstar_grid = interp1d(SMF_z, lgPhiMstar_matrix, axis=0, kind='linear')(z)
    #print(lgPhiMstar_matrix.shape, SMF_z.shape, lgPhiMstar_grid.shape)
    # fix nan
    lgPhiMstar_grid[np.isnan(lgPhiMstar_grid)] = -100
    lgPhiMstar_grid[(lgPhiMstar_grid<-100)] = -100
    # 
    if lgMstar is None:
        return lgMstar_grid, lgPhiMstar_grid
    else:
        return lgPhiMstar_grid



def calc_SMF_Peng2010(z, lgMstar=None, galaxy_type='SFG'):
    # 
    # Peng YingJie 2010
    # IMF: Chabrier 2003
    # Outputs: lgMstar_grid, lgPhiMstar_grid
    # 
    # check z
    if not np.isscalar(z):
        raise ValueError('Please input a float number as the redshift!')
    # 
    # check galaxy_type
    if not (type(galaxy_type) is str):
        raise ValueError('Please input either "ALL", "SFG" or "QG" as the galaxy_type!')
    else:
        if not (galaxy_type in ['SFG', 'QG']):
            raise ValueError('Please input either "ALL", "SFG" or "QG" as the galaxy_type!')
    # 
    # make lgMstar
    if lgMstar is None:
        lgMstar_grid = np.linspace(6.0, 13.0, num=1000, endpoint=True)
    else:
        lgMstar_grid = lgMstar
    # 
    # read SMF
    tb_SMF = Table.read(os.path.dirname(os.path.dirname(__file__))+os.sep+'Data_Tables/datatables_SMF/datatable_PengYingjie2010_SMF_'+galaxy_type+'.txt', format='ascii') # zLo zHi lgMchar  Phi_1   alpha_1  Phi_2  alpha_2
    SMF_zmin = np.min(tb_SMF['zLo'])
    SMF_zmax = np.max(tb_SMF['zHi'])
    # 
    # there is only one redshift bin, but we still check the input z range
    if z < SMF_zmin or z > SMF_zmax:
        raise ValueError('calc_SMF_Peng2010: The input redshift is out of the allowed range of %s -- %s!'%(SMF_zmin, SMF_zmax))
    # 
    # just calculate SMF without interpolation
    SMF_z = (tb_SMF['zLo'].data + tb_SMF['zHi'].data) / 2.0
    SMF_phi_1 = tb_SMF['Phi_1'].data
    SMF_alpha_1 = tb_SMF['alpha_1'].data
    SMF_lgMchar = tb_SMF['lgMchar'].data
    SMF_PhiMstar = Schechter_Function(lgMstar_grid, SMF_phi_1, SMF_lgMchar, SMF_alpha_1) # one component
    if galaxy_type == 'SFG':
        SMF_PhiMstar_SFG = copy(SMF_PhiMstar)
    else:
        SMF_phi_2 = tb_SMF['Phi_2'].data
        SMF_alpha_2 = tb_SMF['alpha_2'].data
        SMF_PhiMstar_SFG = copy(SMF_PhiMstar)
        SMF_PhiMstar_QG = SMF_PhiMstar_SFG + Schechter_Function(lgMstar_grid, SMF_phi_2, SMF_lgMchar, SMF_alpha_2) # two component QG SMF
        SMF_PhiMstar_ALL = SMF_PhiMstar_QG + SMF_PhiMstar_SFG
    # 
    if galaxy_type == 'SFG':
        lgPhiMstar_grid = np.log10(SMF_PhiMstar_SFG)
    elif galaxy_type == 'QG':
        lgPhiMstar_grid = np.log10(SMF_PhiMstar_QG)
    elif galaxy_type == 'ALL':
        lgPhiMstar_grid = np.log10(SMF_PhiMstar_ALL)
    # 
    if lgMstar is None:
        return lgMstar_grid, lgPhiMstar_grid
    else:
        return lgPhiMstar_grid



def calc_SMF_Wright2018_single_component(z, lgMstar=None):
    # 
    # Wright 2018 - SMF - arXiv.1808.01754 - 2018MNRAS.480.3491W.pdf
    # IMF: Chabrier 2003
    # Outputs: lgMstar_grid, lgPhiMstar_grid
    # 
    # check z
    if not np.isscalar(z):
        raise ValueError('Please input a float number as the redshift!')
    # 
    # make lgMstar
    if lgMstar is None:
        lgMstar_grid = np.linspace(6.0, 13.0, num=1000, endpoint=True)
    else:
        lgMstar_grid = lgMstar
    # 
    # 
    SMF_lgMchar = (10.791) + (0.558) * z + (-0.102) * z**2
    SMF_alpha = (-1.160) + (-0.274) * z + (0.028) * z**2
    SMF_phi = 10**((-2.455) + (-0.883) * z + (0.093) * z**2)
    #SMF_rho = (-2.455) + (-0.883) * z + (0.093) * z**2
    #print('z, lgMchar, alpha, phi =', z, SMF_lgMchar, SMF_alpha, SMF_phi)
    SMF_PhiMstar = Schechter_Function(lgMstar_grid, SMF_phi, SMF_lgMchar, SMF_alpha)
    lgPhiMstar_grid = np.log10(SMF_PhiMstar)
    # 
    if lgMstar is None:
        return lgMstar_grid, lgPhiMstar_grid
    else:
        return lgPhiMstar_grid


def calc_SMF_Wright2018_double_component(z, lgMstar=None):
    # 
    # Wright 2018 - SMF - arXiv.1808.01754 - 2018MNRAS.480.3491W.pdf
    # IMF: Chabrier 2003
    # Outputs: lgMstar_grid, lgPhiMstar_grid
    # 
    # check z
    if not np.isscalar(z):
        raise ValueError('Please input a float number as the redshift!')
    # 
    # make lgMstar
    if lgMstar is None:
        lgMstar_grid = np.linspace(6.0, 13.0, num=1000, endpoint=True)
    else:
        lgMstar_grid = lgMstar
    # 
    # 
    SMF_lgMchar = (10.831) + (0.153) * z + (-0.033) * z**2
    SMF_alpha_1 = (-0.579) + (0.048) * z + (0.022) * z**2
    SMF_alpha_2 = (-1.489) + (-0.087) * z + (0.016) * z**2
    SMF_phi_1 = 10**((-2.312) + (-0.658) * z + (0.016) * z**2)
    SMF_phi_2 = 10**((-3.326) + (-0.158) * z + (-0.002) * z**2)
    #print('z, lgMchar, alpha_1, phi_1, alpha_2, phi_2 =', z, SMF_lgMchar, SMF_alpha_1, SMF_phi_1, SMF_alpha_2, SMF_phi_2)
    SMF_PhiMstar = Schechter_Function(lgMstar_grid, SMF_phi_1, SMF_lgMchar, SMF_alpha_1) + \
                   Schechter_Function(lgMstar_grid, SMF_phi_2, SMF_lgMchar, SMF_alpha_2)
    lgPhiMstar_grid = np.log10(SMF_PhiMstar)
    # 
    if lgMstar is None:
        return lgMstar_grid, lgPhiMstar_grid
    else:
        return lgPhiMstar_grid









def calc_Gladders2013_CSFRD(z, t0, tau):
    Gladders2013_t_age = cosmo.age(z).value
    Gladders2013_t0 = float(t0)
    Gladders2013_tau = float(tau)
    Gladders2013_SFR_1 = 1.0/(Gladders2013_t_age*sqrt(2*pi*Gladders2013_tau**2))
    Gladders2013_SFR_2 = exp(-(ln(Gladders2013_t_age)-Gladders2013_t0)**2/(2*Gladders2013_tau**2))
    Gladders2013_SFR = Gladders2013_SFR_1 * Gladders2013_SFR_2
    Gladders2013_SFR = Gladders2013_SFR / 1.64 # converted to Chabrier IMF
    return Gladders2013_SFR


def calc_MadauDickinson2014_CSFRD(z):
    # Madau & Dickinson (2014)
    # Salpeter IMF --> converted to Chabrier IMF
    if type(z) is list:
        z = np.array(z)
    rho_SFR = 0.015 * (1+z)**2.7 / (1.0 + ((1+z)/2.9)**5.6) / 1.64 # converted to Chabrier IMF
    return rho_SFR


def calc_Mstar_integrating_CSFRD_dzliu2018(z):
    # make z_list
    z_list = np.arange(9.75, 0.75, -0.5).tolist()
    z_list.extend([0.75, 0.50, 0.25, 0.125, 0.0625, 0.03125, 0.00000])
    # integrate dzliu2018 CSFRD
    #CSFRD = calc_Gladders2013_CSFRD(z_list, 1.57, 0.67) # Gladder2013 form of CSFRD with Liu et al. (2018) parameters
    CSFRD = calc_MadauDickinson2014_CSFRD(z_list)
    Mstar_cumulated = 0.0
    CSFRD_z_list = []
    CSFRD_Mstar_list = []
    for i in range(len(z_list)-1):
        z_bin = (z_list[i+1] + z_list[i]) / 2.0
        t_bin = cosmo.age(z_list[i+1]).value # Gyr, time since the Big Bang
        time_bin = cosmo.age(z_list[i+1]).value - cosmo.age(z_list[i]).value # Gyr, time interval within the redshift bin
        #mass_loss_time_scale = 1.4 # Myr, BEHROOZI & Silk arXiv:1404.5299
        #mass_loss_time_scale = 3.0 # Myr, Ilbert et al. (2013) PDF page 11 left middle
        mass_loss_time_scale = 0.3 # Myr, Conroy & Wechsler (2009, bibcode 2009ApJ...696..620C) arxiv PDF page 5 Eq (11). 
        Mstar_formed = CSFRD[i] * time_bin * 1e9 # Msun/yr * Gyr * 1e9 => Msun
        Mstar_loss_frac = 0.05 * ln(1.0+(t_bin)/(mass_loss_time_scale*1e-3)) 
                                # see Ilbert et al. (2013) PDF page 11 left middle; Conroy & Wechsler (2009) arxiv PDF page 5 Eq (11). 
                                # see https://arxiv.org/pdf/1404.5299.pdf PDF page 3 Eq (6); Conroy & Wechsler (2009) arxiv PDF page 5 Eq (11). 
        Mstar_cumulated += Mstar_formed * (1.0 - Mstar_loss_frac) # total new star created, locked in stellar mass
        CSFRD_z_list.append(z_bin)
        CSFRD_Mstar_list.append(Mstar_cumulated)
    CSFRD_z_list = np.array(CSFRD_z_list)[::-1]
    CSFRD_Mstar_list = np.array(CSFRD_Mstar_list)[::-1]
    # spline at z for the output
    Mstar_cumulated_at_z = 10**(InterpolatedUnivariateSpline(CSFRD_z_list, np.log10(CSFRD_Mstar_list), k=1)(z))
    return Mstar_cumulated_at_z



def calc_SMF_dzliu2018(z=None, lgMstar=None, galaxy_type='SFG', z_list=None):
    # 
    # dzliu 2018 - my own choice
    # IMF: Chabrier 2003
    # Outputs: lgMstar_grid, lgPhiMstar_grid
    # 
    # check z
    if z is not None:
        if not np.isscalar(z):
            raise ValueError('Please input a float number as the redshift!')
    # 
    # check galaxy_type
    if not (type(galaxy_type) is str):
        raise ValueError('Please input either "ALL", "SFG" or "QG" as the galaxy_type!')
    else:
        if not (galaxy_type in ['SFG', 'QG', 'ALL']):
            raise ValueError('Please input either "ALL", "SFG" or "QG" as the galaxy_type!')
    # 
    # make lgMstar
    if lgMstar is None:
        lgMstar_grid = np.linspace(6.0, 13.0, num=1000, endpoint=True)
    else:
        lgMstar_grid = lgMstar
    # 
    # code from '/Volumes/GoogleDrive/Team Drives/DeepFields/Simulations/Cosmological_Galaxy_Modelling/a_dzliu_code_Plot_SMF_dzliu_model.sm'
    # make z_list
    if z_list is None:
        z_list = np.arange(9.75, 0.75, -0.5).tolist()
        z_list.extend([0.75, 0.50, 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.00000])
        #z_list = []
        #z_list.extend(np.arange(9.75, 4.00, -0.5).tolist())
        #z_list.extend(np.arange(4.00, 3.00, -0.25).tolist())
        #z_list.extend(np.arange(3.00, 2.00, -0.125).tolist())
        #z_list.extend(np.arange(2.00, 1.00, -0.0625).tolist())
        #z_list.extend(np.arange(1.00, 0.00, -0.03125).tolist())
        #z_list.extend([0.00])
    SMF_z_list = []
    SMF_Phi_list = []
    for i in range(len(z_list)-1):
        z_bin = (z_list[i] + z_list[i+1]) / 2.0
        t_bin = cosmo.age(z_bin).value # Gyr since the Big Bang
        Schechter_M = 10**lgMstar_grid
        Schechter_Mstep = lgMstar_grid[1] - lgMstar_grid[0]
        # 
        # first construct SMF at each redshift bin
        #if False:
        #    EEE_Mchar_MQG = lg((10**10.68)*(t_bin/cosmo.age(0).value)**(0.50)) # Quiescent galaxies' SMF's first component
        #    EEE_Mchar_NQG = lg((10**10.68)*(t_bin/cosmo.age(0).value)**(0.50)) # Quiescent galaxies' SMF's second component
        #    EEE_Mchar_SFG = lg((10**10.72)*(t_bin/cosmo.age(0).value)**(-0.50)) # Star-forming galaxies' SMF
        #    EEE_Phi_MQG = (3.400e-3)*(t_bin/cosmo.age(0).value)**(4.00)
        #    EEE_Phi_NQG = (0.126e-3)*(t_bin/cosmo.age(0).value)**(4.00) # decrease much faster with increasing z
        #    EEE_Phi_SFG = (0.900e-3)*(t_bin/cosmo.age(0).value)**(0.20)
        #    EEE_alpha_MQG = -0.39 + (z_bin)*(1.00)
        #    EEE_alpha_NQG = -1.56 + (z_bin)*(1.00)
        #    EEE_alpha_SFG = -1.40 + (z_bin)*(-0.06) # slope -- this makes too many low mass galaxiest at z>4 -- faint-end of the 24um number counts -- no, this is because stellar SED has too old age, f24um is underestimated!
        #    #EEE_alpha_SFG = -1.40 + (z)*(-0.03) # slope
        #    Schechter_P_MQG = Schechter_Function(lgMstar_grid, EEE_Phi_MQG, EEE_Mchar_MQG, EEE_alpha_MQG)
        #    Schechter_P_NQG = Schechter_Function(lgMstar_grid, EEE_Phi_NQG, EEE_Mchar_NQG, EEE_alpha_NQG)
        #    Schechter_P_SFG = Schechter_Function(lgMstar_grid, EEE_Phi_SFG, EEE_Mchar_SFG, EEE_alpha_SFG)
        #    Schechter_P_QG = Schechter_P_MQG + Schechter_P_NQG
        #    #print('z = %.04f, lgSchechter_P_SFG = %s, lgSchechter_P_QG = %s'%(z_bin, np.log10(Schechter_P_SFG), np.log10(Schechter_P_QG)))
        # 
        if z_bin < 0.02:
            Schechter_P_QG = 10**(calc_SMF_Peng2010(0.02, lgMstar=lgMstar_grid, galaxy_type='QG'))
            Schechter_P_SFG = 10**(calc_SMF_Peng2010(0.02, lgMstar=lgMstar_grid, galaxy_type='SFG'))
        elif z_bin < 0.085:
            Schechter_P_QG = 10**(calc_SMF_Peng2010(z_bin, lgMstar=lgMstar_grid, galaxy_type='QG'))
            Schechter_P_SFG = 10**(calc_SMF_Peng2010(z_bin, lgMstar=lgMstar_grid, galaxy_type='SFG'))
        elif z_bin < 0.2:
            Schechter_P_QG = 10**(calc_SMF_Peng2010(0.085, lgMstar=lgMstar_grid, galaxy_type='QG')) * (1.0-((0.2-0.085)-(0.2-z_bin))/(0.2-0.085)) + 10**(calc_SMF_Davidzon2017(0.2, lgMstar=lgMstar_grid, galaxy_type='QG')) * (0.0+((0.2-0.085)-(0.2-z_bin))/(0.2-0.085))
            Schechter_P_SFG = 10**(calc_SMF_Peng2010(0.085, lgMstar=lgMstar_grid, galaxy_type='SFG')) * (1.0-((0.2-0.085)-(0.2-z_bin))/(0.2-0.085)) + 10**(calc_SMF_Davidzon2017(0.2, lgMstar=lgMstar_grid, galaxy_type='SFG')) * (0.0+((0.2-0.085)-(0.2-z_bin))/(0.2-0.085))
        elif z_bin < 4.0:
            Schechter_P_QG = 10**(calc_SMF_Davidzon2017(z_bin, lgMstar=lgMstar_grid, galaxy_type='QG'))
            Schechter_P_SFG = 10**(calc_SMF_Davidzon2017(z_bin, lgMstar=lgMstar_grid, galaxy_type='SFG'))
        else:
            Schechter_P_QG = 10**(calc_SMF_Davidzon2017(4.0, lgMstar=lgMstar_grid, galaxy_type='QG'))
            Schechter_P_SFG = 10**(calc_SMF_Davidzon2017(4.0, lgMstar=lgMstar_grid, galaxy_type='SFG'))
        # 
        # 
        # then re-normalize SMF to total stellar mass integrated from CSFRD
        Mstar_cumulated = calc_Mstar_integrating_CSFRD_dzliu2018(z_bin)
        #CSFRD_at_z_bin = calc_MadauDickinson2014_CSFRD(z_bin)
        Schechter_P_int = (Schechter_P_QG+Schechter_P_SFG)*Schechter_M
        Schechter_M_total = sum(Schechter_P_int*Schechter_Mstep) # *Schechter_Mstep to account for Psi unit [dex-1]
        Schechter_P_SFG = Schechter_P_SFG * (Mstar_cumulated / Schechter_M_total)
        Schechter_P_QG = Schechter_P_QG * (Mstar_cumulated / Schechter_M_total)
        Schechter_P_ALL = Schechter_P_SFG + Schechter_P_QG
        print('z = %.04f, lgMstar_CSFRD = %0.2f, lgMstar_SMF = %0.2f, renorm = %s'%(z_bin, np.log10(Mstar_cumulated), np.log10(Schechter_M_total), (Mstar_cumulated / Schechter_M_total)))
        #print('z = %.04f, lgCSFRD = %0.2f, lgMstar_CSFRD = %0.2f, lgMstar_SMF = %0.2f'%(z_bin, np.log10(CSFRD_at_z_bin), np.log10(Mstar_cumulated), np.log10(Schechter_M_total)))
        # 
        # --> checked OK
        # z = 0.0156, lgCSFRD = -2.04, lgMstar_CSFRD = 8.40, lgMstar_SMF = 8.35
        # 
        # 
        SMF_z_list.append(z_list[i+1]) # append the lower redshift end point of each bin
        if galaxy_type == 'SFG':
            SMF_Phi_list.append(Schechter_P_SFG)
        elif galaxy_type == 'QG':
            SMF_Phi_list.append(Schechter_P_QG)
        elif galaxy_type == 'ALL':
            SMF_Phi_list.append(Schechter_P_ALL)
        # 
    # spline at z for the output
    SMF_z_list = np.array(SMF_z_list)[::-1] # make z increasing order
    SMF_Phi_list = np.array(SMF_Phi_list)[::-1].T # make z increasing order
    #print(SMF_z_list.shape, SMF_Phi_list.shape)
    if z is None:
        lgPhiMstar_matrix = np.log10(SMF_Phi_list.T)
        return SMF_z_list, lgMstar_grid, lgPhiMstar_matrix
    else:
        lgPhiMstar_grid = interp1d(SMF_z_list, np.log10(SMF_Phi_list), kind='cubic')(z) # kind='nearest' 'linear' 'quadratic' 'cubic'
    # 
    if lgMstar is None:
        return lgMstar_grid, lgPhiMstar_grid
    else:
        return lgPhiMstar_grid

























#!/usr/bin/env python
# 
# Usage:
#    from calc_delta_GD import ( calc_deltaGD_from_metalZ_following_Leroy2011, 
#                                calc_deltaGD_from_metalZ_following_Magdis2012, 
#                                calc_deltaGD_from_metalZ_following_RemyRuyer2014a, 
#                                calc_deltaGD_from_metalZ_following_RemyRuyer2014b, 
#                                calc_deltaGD_from_metalZ_following_Genzel2015, 
#                              )
# 
# 

from __future__ import print_function

import os, sys, re, json, time, astropy
import numpy as np
from astropy.table import Table, Column, hstack
from copy import copy

if not (os.path.dirname(os.path.abspath(__file__)) in sys.path): sys.path.append(os.path.dirname(os.path.abspath(__file__)))
import apply_cosmology
cosmo = apply_cosmology.cosmo

if sys.version_info.major >= 3:
    long = int
else:
    pass



# 
# def 
# 
def calc_deltaGD_from_metalZ_following_Leroy2011(metalZ):
    # Leroy, 2011ApJ...737...12L
    # Sect. 5.1 Paragraph 2
    # metalZ is 12+log(O/H)_{PP04}
    # deltaGD = Mgas/Mdust
    # here Mgas inclues HI + H2 + Helium, i.e., atomic + mol. gas
    # They assume that δGDR does not vary between the atomic and molecular ISM. 
    return np.power(10.0, (9.4) - (0.85) * metalZ )


def calc_deltaGD_from_metalZ_following_Magdis2012(metalZ):
    # Magdis, 2012ApJ...760....6M
    # Sect. 4.2 Paragraph 1
    # 
    # metalZ is 12+log(O/H)_{PP04}
    # deltaGD = Mgas/Mdust, here Mgas includes HI + H2 + Helium
    return np.power(10.0, (10.54) - (0.99) * metalZ )


def calc_deltaGD_from_metalZ_following_RemyRuyer2014a(metalZ):
    # Remy-Ruyer et al. 2014 (doi:10.1051/0004-6361/201322803), broken power law, high metallicity part. See their Table 1. 
    #   catalog data: http://vizier.u-strasbg.fr/viz-bin/VizieR-3?-source=J/A%2bA/582/A121/sample
    # XCO,MW case
    # includes HI + H2 + He + gaseous metal mass (= Z[O/H] * Mgas)
    #   Mgas = MHI + MH2 + MHe = 1.38 * (MHI + MH2)
    #   Z[O/H]_solar = 0.014 (Asplund et al. 2009)
    #   metalZ = 12+log(O/H)
    #   metalZ_solar = 8.69 (Asplund et al. 2009)
    # one example
    #   SBS 1533+574, metalZ = 8.05, MH2_alphaCO_MW = 3.70e7, MH2_alphaCO_Z = 2.03e9, Mdust.= 10**5.89, G/D = 47 - 2615. 
    #   NGC 1569, metalZ = 8.02, MH2_alphaCO_MW = 7.08e5, MH2_alphaCO_Z = 1.54e7, Mdust = 10**5.56, G/D = 2 - 42. 
    a = 2.21
    aH = 1.00
    b = 0.68
    aL = 3.08
    metaZ = metalZ
    metaZknee = 7.96
    metaZsolar = 8.69
    if ~np.isscalar(metaZ):
        maskZ = (metaZ>metaZknee)
        GDR_ISM = metaZ * 0.0
        GDR_ISM[maskZ] = 10**(a + aH*(metaZsolar-metaZ[maskZ]))
        GDR_ISM[~maskZ] = 10**(b + aL*(metaZsolar-metaZ[~maskZ]))
    else:
        GDR_ISM = 10**(a + aH*(metaZsolar-metaZ)) if metaZ>7.96 else 10**(b + aL*(metaZsolar-metaZ))
    #GDR_mol = GDR_ISM/1.38 # considering only molecular gas (without HI, but still with Helium). See Remy-Ruyer et al. 2014 Eq.(3)
    return GDR_ISM


def calc_deltaGD_from_metalZ_following_RemyRuyer2014b(metalZ):
    # Remy-Ruyer et al. 2014 (doi:10.1051/0004-6361/201322803), broken power law, high metallicity part. See their Table 1. 
    # XCO,Z case
    # includes HI + H2 + He + gaseous metal mass (= Z[O/H] * Mgas)
    #   Mgas = MHI + MH2 + MHe = 1.38 * (MHI + MH2)
    #   Z[O/H]_solar = 0.014 (Asplund et al. 2009)
    #   metalZ = 12+log(O/H)
    #   metalZ_solar = 8.69 (Asplund et al. 2009)
    a = 2.21
    aH = 1.00
    b = 0.96
    aL = 3.10
    metaZ = metalZ
    metaZknee = 8.10
    metaZsolar = 8.69
    if ~np.isscalar(metaZ):
        maskZ = (metaZ>metaZknee)
        GDR_ISM = metaZ * 0.0
        GDR_ISM[maskZ] = 10**(a + aH*(metaZsolar-metaZ[maskZ]))
        GDR_ISM[~maskZ] = 10**(b + aL*(metaZsolar-metaZ[~maskZ]))
    else:
        GDR_ISM = 10**(a + aH*(metaZsolar-metaZ)) if metaZ>7.96 else 10**(b + aL*(metaZsolar-metaZ))
    #GDR_mol = GDR_ISM/1.38 # considering only molecular gas (without HI, but still with Helium). See Remy-Ruyer et al. 2014 Eq.(3)
    return GDR_ISM


def calc_deltaGD_from_metalZ_following_Genzel2015(metalZ):
    # Genzel et al. 2015 (doi:10.1088/0004-637X/800/1/20), Eq. 10
    #   Following Magdis et al. (2012a) and Magnelli et al. (2012a) we converted the Draine & Li (2007) 
    #   model dust masses to (molecular) gas masses by applying the metallicity dependent dust-to-gas 
    #   ratio fitting function for z ∼ 0 SFGs found by Leroy et al. (2011):
    #return 1.0/10**(-2. + 0.85*(metalZ-8.67) ) # almost exactly the same as Leroy2011, note that they express this as dust/gas instead of gas/dust.
    return calc_deltaGD_from_metalZ_following_Leroy2011(metalZ)











#!/usr/bin/env python
# 
# Usage:
#    from calc_alpha_CO import ( calc_metalZ_from_FMR_following_Mannucci2010, 
#                                calc_alphaCO_from_metalZ_following_Genzel2015a, 
#                                calc_alphaCO_from_metalZ_following_Genzel2015b )
# 

from __future__ import print_function

import os, sys, re, json, time, astropy
import numpy as np
from astropy.table import Table, Column, hstack
from copy import copy

if not (os.path.dirname(os.path.abspath(__file__)) in sys.path): sys.path.append(os.path.dirname(os.path.abspath(__file__)))
import apply_cosmology
cosmo = apply_cosmology.cosmo

if sys.version_info.major >= 3:
    long = int
else:
    pass



# 
# def 
# 
def calc_alphaCO_from_metalZ_following_Wilson1995(metalZ):
    # Wilson, 1995ApJ...448L..97W
    # Eq. (4)
    metalZ_solar_old = 8.88
    metalZ_solar_new = 8.69
    XCO_Galactic = 3e20
    alphaCO_Galactic = 3e20/2e20*4.3
    return np.power(10.0, 5.95 - 0.67 * (metalZ-metalZ_solar_new+metalZ_solar_old)) * alphaCO_Galactic


def calc_alphaCO_from_metalZ_following_Magdis2012(metalZ):
    # Magdis, 2012ApJ...760....6M
    # Eq. (8)
    return np.power(10.0, (12.8) - (1.39) * metalZ )


def calc_alphaCO_from_metalZ_following_Genzel2012(metalZ):
    # Genzel 2012
    # Eq. (7) or Fig. 5 label
    return np.power(10.0, (12.0) - (1.3) * metalZ )
    #return np.power(10.0, (11.8) - (1.27) * metalZ )


def calc_alphaCO_from_metalZ_following_Genzel2015a(metalZ, metalZ_solar = 8.67, alphaCO_MilkyWay = 4.36):
    # Genzel 2015ApJ...800...20G
    # Eq. (6)
    # 
    # alphaCO_MilkyWay 4.36 ± 0.9 (M⊙ (K km s−1 pc2 )−1 ), 
    #   which is equivalent to XCO = N(H2)/(TRJ=1∆v) = 2 × 10^20 (cm−2 (K km s−1)−1; 
    #   Strong & Mattox 1996; Dame et al. 2001; Grenier et al. 2005; 
    #   Bolatto et al. 2008, 2013; Leroy et al. 2011; Abdo et al. 2010; 
    #   Ostriker et al. 2010).
    # 
    return 0.67 * np.exp(0.36 * np.power(10.0, -(metalZ-metalZ_solar)) ) * alphaCO_MilkyWay


def calc_alphaCO_from_metalZ_following_Genzel2015b(metalZ, metalZ_solar = 8.67, alphaCO_MilkyWay = 4.36):
    # Genzel 2015ApJ...800...20G
    # Eq. (7)
    # 
    # alphaCO_MilkyWay 4.36 ± 0.9 (M⊙ (K km s−1 pc2 )−1 ), 
    #   which is equivalent to XCO = N(H2)/(TRJ=1∆v) = 2 × 10^20 (cm−2 (K km s−1)−1; 
    #   Strong & Mattox 1996; Dame et al. 2001; Grenier et al. 2005; 
    #   Bolatto et al. 2008, 2013; Leroy et al. 2011; Abdo et al. 2010; 
    #   Ostriker et al. 2010).
    # 
    return np.power(10.0, -1.27 * (metalZ-metalZ_solar) ) * alphaCO_MilkyWay


def calc_alphaCO_from_metalZ_following_Genzel2015(metalZ, metalZ_solar = 8.67, alphaCO_MilkyWay = 4.36):
    return calc_alphaCO_from_metalZ_following_Genzel2015a(metalZ, metalZ_solar = metalZ_solar, alphaCO_MilkyWay = alphaCO_MilkyWay)


def calc_alphaCO_from_metalZ_following_Bolatto2013(metalZ, Sigma_total, Sigma_GMC_100 = 1.0, metalZ_solar = 8.67, alphaCO_MilkyWay = 4.36):
    # Bolatto, Wolfire, Leroy - 2013 - The CO-to-H2 Conversion Factor.pdf
    # Eq. (31) on PDF page 55.
    # 
    # Sigma_GMC_100 is the mean GMC gas surface density in unit of 100 Msolar pc^{-2}. 
    # Sigma_total is the total gas surface density in unit of 1 Msolar pc^{-2}. 
    # t_gamma is γ ≈ 0.5 for 􏰣total > 100 M⊙ pc−2 and γ = 0 otherwise
    if np.isscalar(Sigma_total):
        if Sigma_total >= 100.0:
            t_gamma = 0.5
        else:
            t_gamma = 0.0
    else:
        t_gamma = np.array(Sigma_total) * 0.0
        t_gamma[(Sigma_total>=100.0)] = 0.5
    return 2.9 * np.exp(0.4/(np.power(10.0,metalZ-metalZ_solar)*Sigma_GMC_100)) * np.power(Sigma_total/100.0,-t_gamma)


def calc_alphaCO_from_metalZ_following_Accurso2017(metalZ, DeltaMS = 0.0):
    # Accurso2017 calibrated alphaCO with L_CII/L_CO
    # see their Eq.(24)
    return np.power(10.0, 14.752 - 1.623 * metalZ + 0.062 * DeltaMS)


def calc_alphaCO_from_metalZ_following_Bertemes2018(metalZ):
    # Bertemes2018 used the geometric mean of Genzel2012 and Bolatto2013 alphaCO
    # see their Eq.(9),(10)
    # They assumed 
    alphaCO_G12 = np.power(10.0, -1.27 * (metalZ-8.67) )
    alphaCO_B13 = 0.67 * np.exp(0.36 * (-(metalZ-8.67)) )
    alphaCO_MW = 3.2
    alphaCO_combined = alphaCO_MW * np.sqrt(alphaCO_G12*alphaCO_B13) * 1.36
    return alphaCO_combined


def calc_alphaCO_from_metalZ_following_Tacconi2018(metalZ):
    # Tacconi2018 used the geometric mean of Genzel2012 and Bolatto2013 alphaCO (see also Equations (6) and (7) in Genzel2015)
    # see their Eq.(2)
    # They assumed 
    alphaCO_G12 = np.power(10.0, -1.27 * (metalZ-8.67) )
    alphaCO_B13 = 0.67 * np.exp(0.36 * (-(metalZ-8.67)) )
    alphaCO_MW = 4.36 # Helium included
    alphaCO_combined = alphaCO_MW * np.sqrt(alphaCO_G12*alphaCO_B13)
    return alphaCO_combined


def calc_alphaCO_from_metalZ_following_Boselli2014(lgL_Hband):
    # Boselli2014a (2014A&A...564A..65B) used a luminosity dependent alphaCO
    # see their page 2, paragraph 2.
    # 
    X_CO = np.power(10.0, -0.38 * lgL_Hband + 24.23 )
    alphaCO = X_CO / 2.0e20 * 4.3
    return alphaCO










#!/usr/bin/env python
# 

from __future__ import print_function

import os, sys, time, re

if sys.version_info.major >= 3:
    long = int
else:
    pass


# 
# prepare function for cross-matching
# 
def flatten(item, keepcls=(), keepobj=()):
    """ Flatten a list, 
        see -- https://stackoverflow.com/questions/2158395/flatten-an-irregular-list-of-lists
    """
    if not hasattr(item, '__iter__') or isinstance(item, keepcls) or item in keepobj:
        yield item
    else:
        for i in item:
            for j in flatten(i, keepcls, keepobj + (item,)):
                yield j


def search_for_matches_in_a_sorted_array(input_array, match_value, start_position = 0, search_direction = +1, output_allmatches = False):
    """ Example: xmatch2 = search_for_matches_in_a_sorted_array(array2, array1[i1], i2, -1)
    """
    i = start_position
    xmatches = []
    while (i >= 0 and i <= len(input_array)-1):
        val = input_array[i]
        if val == match_value:
            xmatches.append(i)
            if not output_allmatches:
                break
        else:
            if search_direction > 0:
                if val > match_value:
                    break
            elif search_direction < 0:
                if val < match_value:
                    break
        i = i + search_direction
    return xmatches


def cross_match_sorted_arrays(input_array_list, output_allmatches = False, output_nonmatches = False):
    """ Return two index array with common items in the two arrays
        We will search for only one match per item, unless the option 'output_allmatches' is set to True. 
        We have the option 'output_nonmatches' to also output all non-matches.
        TODO: what if input_array has duplicates?
    """ 
    xmatches = [] # cross-matched index array for each input_array item
    nonmatches = [] # non-matched index array for each input_array item
    i = []
    if not (type(input_array_list) is list):
        print('Error! The input_array_list should be a list!')
        sys.exit()
    if len(input_array_list) == 0:
        print('Error! The input_array_list should be a non-empty list!')
        sys.exit()
    for input_array in input_array_list:
        if len(input_array) == 0:
            print('Error! The input_array should be non-empty!')
            sys.exit()
        xmatches.append([])
        nonmatches.append([])
        i.append(0) # i[0] = 0
    #i = [0]*len(input_array_list)
    while i[0] < len(input_array_list[0]):
        # take the first input array as the reference array
        input_array_1 = input_array_list[0]
        i1 = i[0]
        val1 = long(input_array_1[i1])
        countxmatches = 1 # account for input_array_1 itself
        if output_allmatches:
            tmpxmatches = [[i1]] # a list of lists, each item is a list of matched item indexes for each input array.
        else:
            tmpxmatches = [i1] # when not outputing all matches, this is a list of index
        # try to find matches in each other input array
        for j in range(1,len(input_array_list)):
            input_array_2 = input_array_list[j]
            i2 = i[j]
            tmpxmatches2 = []
            tmpxmatches2a = search_for_matches_in_a_sorted_array(input_array_2, val1, i2, -1, output_allmatches = output_allmatches)
            if len(tmpxmatches2a) > 0:
                tmpxmatches2.extend(tmpxmatches2a)
            if (len(tmpxmatches2a) == 0 or output_allmatches):
                # if we already got matches and output_allmatches is True, then we stop finding new matches toward the other direction
                tmpxmatches2b = search_for_matches_in_a_sorted_array(input_array_2, val1, i2+1, +1, output_allmatches = output_allmatches)
                if len(tmpxmatches2b) > 0:
                    tmpxmatches2.extend(tmpxmatches2b)
            if len(tmpxmatches2) > 0:
                countxmatches = countxmatches + 1
                if output_allmatches:
                    tmpxmatches.append(tmpxmatches2)
                else:
                    tmpxmatches.append(tmpxmatches2[0]) # take only one match for each item, ignoring other matches
        # if found matches in all input arrays
        if countxmatches == len(input_array_list):
            xmatches[0].append(tmpxmatches[0]) # which is just i1 or [i1]
            for j in range(1,len(input_array_list)):
                xmatches[j].append(tmpxmatches[j])
                if output_allmatches:
                    i[j] = max(tmpxmatches[j]) + 1 # here we increase one because all input arrays are sorted
                else:
                    i[j] = tmpxmatches[j] + 1 # here we increase one because all input arrays are sorted
        # 
        i[0] = i[0] + 1
        #print('i[0] =', i[0])
    # 
    if output_nonmatches:
        input_array_1 = input_array_list[0]
        xmatches1 = list(flatten(xmatches[0]))
        allindex1 = range(len(input_array_1))
        nonmatch1 = list(set(allindex1)-set(xmatches1))
        nonmatches[0] = nonmatch1
        for j in range(1,len(input_array_list)):
            input_array_2 = input_array_list[j]
            xmatches2 = list(flatten(xmatches[j]))
            allindex2 = range(len(input_array_2))
            nonmatch2 = list(set(allindex2)-set(xmatches2))
            nonmatches[j] = nonmatch2
        return xmatches, nonmatches
    # 
    return xmatches





name = 'a3cosmos_gas_evolution'





# 
# import functions
# 
from .Common_Python_Code import calc_galaxy_main_sequence
from .Common_Python_Code import calc_gas_fraction
from .Common_Python_Code import calc_gas_depletion_time
from .Common_Python_Code import apply_cosmology
cosmo = apply_cosmology.cosmo






from .__main__ import help

from .__main__ import load_all_modules

load_all_modules()

from .__main__ import *




#!/usr/bin/env python
# 
# Usage:
#    from calc_fmol import ( calc_fmol_from_metalZ_following_Krumholz2009, 
#                            calc_fmol_from_metalZ_following_Dave2016, 
#                            calc_fmol_from_metalZ_following_Popping2014 )
# 
# 

from __future__ import print_function

import os, sys, re, json, time, astropy
import numpy as np
from astropy.table import Table, Column, hstack
from copy import copy

from astropy.cosmology import FlatLambdaCDM
cosmo = FlatLambdaCDM(H0=73, Om0=0.27, Tcmb0=2.725)

if sys.version_info.major >= 3:
    long = int
else:
    pass



# 
# def 
# 
def calc_fmol_from_metalZ_following_Krumholz2009(metalZOH):
    # 
    # compute fmol = M_H2/(M_HI+M_H2) following
    # -- Krumholz 2009 ApJ 699 850
    # -- Eq.(2)
    # 
    metalZ_solar = 8.69
    metalZ_ = 10**(metalZOH - metalZ_solar) # in unit of metalZ_solar
    Sigma_total_gas = 30.0 # 70.0 #<TODO># Schruba 2011 Fig.13
    ksi_KMT09 = 0.77 * (1. + 3.1 * np.power(metalZ_, 0.365))
    s_ = np.log(1. + 0.6 * ksi_KMT09) / (0.04 * Sigma_total_gas * metalZ_)
    delta_ = 0.0712 * (0.1 * s_**(-1) + 0.675)**(-2.8)
    f_H2_to_total = 1.0 - np.power(1.0 + (3./4. * (s_)/(1.+delta_))**(-5), -1./5.)
    return f_H2_to_total


def calc_fmol_from_metalZ_following_Dave2016(metalZOH):
    # 
    # compute fmol = M_H2/(M_HI+M_H2) following
    # -- Dave 2016 - cosmological hydrodynamic simulation MUFASA - 2016MNRAS.462.3265D.pdf
    # -- Eq.(1,2)
    # 
    metalZ_solar = 8.69
    metalZ_ = 10**(metalZOH - metalZ_solar)
    Sigma_total_gas = 30.0 # 70.0 #<TODO># Schruba 2011 Fig.13
    ksi_KMT09 = 0.77 * (1. + 3.1 * np.power(metalZ_, 0.365))
    s_ = np.log(1. + 0.6 * ksi_KMT09 + 0.01 * ksi_KMT09**2) / (0.0396 * metalZ_ * Sigma_total_gas)
    f_H2_to_total = 1.0 - 0.75 * s_ / (1.0 + 0.25 * s_)
    return f_H2_to_total


def calc_fmol_from_metalZ_following_Popping2014(metalZOH, U_MW = None):
    # 
    # compute fmol = M_H2/(M_HI+M_H2) following
    # -- http://www.astro.rug.nl/~gpopp/popping.samgas.pdf
    # -- Eq.(8)
    # 
    metalZ_solar = 8.69
    if U_MW is None:
        U_MW = 10**(5.0 * (metalZOH - metalZ_solar)) # UV background in units of MW UV ISRF <TODO>
        U_MW[U_MW<1.0] = 1.0
    if np.isscalar(U_MW):
        U_MW = np.array([U_MW]*len(metalZOH))
    D_MW = 10**(metalZOH - metalZ_solar)
    D_star = 1.5e-3 * np.log(1. + np.power(3.*U_MW,1.7))
    alpha_ = 5. * (U_MW/2.) / (1. + (U_MW/2.)**2)
    s_ = 0.04 / (D_star + D_MW)
    g_ = (1. + alpha_ * s_ + s_**2) / (1. + s_)
    Gamma_ = np.log(1. + g_ * np.power(D_MW,3./7.) * np.power(U_MW/15.,4./7.))
    Sigma_tide = 20. * np.power(Gamma_,4./7.) / D_MW / np.sqrt(1.+U_MW*D_MW**2)
    Sigma_total_gas_r = 10.0 # 70.0 #<TODO># Schruba 2011 Fig.13
    f_H2_r = (1. + Sigma_tide / (Sigma_total_gas_r))**(-2)
    #f_H2_to_HI = f_H2_r
    #f_H2_to_total = 1./(1.+1./f_H2_to_HI)
    f_H2_to_total = f_H2_r
    print('-----')
    for i in range(len(f_H2_to_total)):
        print('metalZ 12+log10(O/H) %0.3f, f_H2_to_total %0.6f, D_star %0.6f, D_MW %0.6f, U_MW %0.6f'%(metalZOH[i], f_H2_to_total[i], D_star[i], D_MW[i], U_MW[i]))
    return f_H2_to_total











#!/usr/bin/env python
# coding=utf-8
# 
# to build:
#   python setup.py sdist bdist_wheel
# 

from setuptools import setup, find_packages

setup(
    name='a3cosmos_gas_evolution',
    version=0.9,
    description=(
        'A Python Package for Galaxy Cold Molecular Gas and Star Formation Evolution Equations.'
    ),
    keywords="a3cosmos galaxy molecular gas dust star formation SFR evolution equation parametrization",
    long_description=open('README.rst').read(),
    author='Daizhong Liu & A3COSMOS Collaboration',
    author_email='dzliu@mpia.de',
    maintainer='Daizhong Liu & A3COSMOS Collaboration',
    maintainer_email='dzliu@mpia.de',
    license='BSD License',
    packages=find_packages(),
    platforms=["all"],
    url='https://sites.google.com/view/a3cosmos',
    project_urls={
        #"Bug Tracker": "https://bugs.example.com/HelloWorld/",
        "Documentation": "https://sites.google.com/view/a3cosmos/code-a3cosmos-gas-evolution",
        #"Source Code": "https://code.example.com/HelloWorld/",
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        #'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Scientific/Engineering :: Astronomy',
    ],
    install_requires=[
        'numpy',
        'astropy',
        'matplotlib',
    ], 
    include_package_data=True,
    #package_data={
    #    'project': ['default_data.json', 'other_datas/default/*.json']
    #},
    #data_files=
)






from __future__ import print_function # for Python<=2.X
#from __future__ import with_statement # for Python<=2.5
#from __future__ import nested_scopes # for Python<=2.1
#from pkg_resources import parse_version
from distutils.version import LooseVersion, StrictVersion
import os, sys, re
import inspect



# 
# import functions (keep these the same as in __init__.py)
# 
from .Common_Python_Code import calc_galaxy_main_sequence
from .Common_Python_Code import calc_gas_fraction
from .Common_Python_Code import calc_gas_depletion_time
from .Common_Python_Code import apply_cosmology
cosmo = apply_cosmology.cosmo




# 
# load all modules
# 
def load_all_modules():
    load_all_functions_in_one_module(calc_galaxy_main_sequence, r'^calc_SFR_MS_.*',             print_to_screen = False)
    load_all_functions_in_one_module(calc_gas_fraction,         r'^calc_gas_fraction_.*',       print_to_screen = False)
    load_all_functions_in_one_module(calc_gas_depletion_time,   r'^calc_gas_depletion_time_.*', print_to_screen = False)




# 
# load and print all functions in one module
# 
def load_all_functions_in_one_module(t_module, func_name_pattern = '', print_to_screen = False):
    # get function list in the input t_module
    t_func_list = [ t_func_name \
                    for t_func_name in dir(t_module) \
                        if re.match(func_name_pattern, t_func_name, re.IGNORECASE) \
                            and hasattr(getattr(t_module, t_func_name), '__call__') ]
    # sort function names
    t_func_list.sort(key=LooseVersion)
    # prepare printing prefix
    t_print_prefix = '    '
    # get function name maximum string length
    t_func_nchar = 0
    for t_func_name in t_func_list:
        t_func_nchar = max([len(t_func_name), t_func_nchar])
    # loop function names and print function arguments
    for t_func_name in t_func_list:
        if print_to_screen == True:
            print(t_print_prefix, end='')
            print(('%%-%ds'%(t_func_nchar))%(t_func_name), end='')
        t_func_self = getattr(t_module,t_func_name)
        # 
        # The commented code below are for getting t_func_args without import inspect. The problem is len(t_func_defaults) != t_func_narg. 
        #if sys.version_info.major < 2 or (sys.version_info.major==2 and sys.version_info.minor<=5):
        #    t_func_narg = t_func_self.func_code.co_argcount
        #    t_func_args = t_func_self.func_code.co_varnames
        #    t_func_defaults = t_func_self.func_defaults
        #else:
        #    t_func_narg = t_func_self.__code__.co_argcount
        #    t_func_args = t_func_self.__code__.co_varnames
        #    t_func_defaults = t_func_self.__defaults__
        # 
        # Get t_func_args with inspect. Still has the problem that len(t_func_defaults) != t_func_narg. 
        #t_func_spec = inspect.getargspec(t_func_self) # inspect.getargspec() is deprecated since Python 3.0, use inspect.signature() or inspect.getfullargspec()
        #t_func_spec = inspect.getfullargspec(t_func_self)
        #t_func_narg = len(t_func_spec.args)
        #t_func_args = t_func_spec.args
        #t_func_defaults = t_func_spec.defaults
        # 
        # Loop function arguments and print
        #for t_func_iarg in range(t_func_narg):
        #    if t_func_iarg == 0: print('(', end='')
        #    else: print(', ', end='')
        #    print(t_func_args[t_func_iarg], end='')
        #    if t_func_defaults[t_func_iarg] != None:
        #        print(' = %s'%(t_func_defaults[t_func_iarg]), end='')
        #    print(')', end='')
        # 
        # Directly print str(inspect.signature()) is the new way
        if print_to_screen == True:
            print(' ', end='')
            print(str(inspect.signature(t_func_self)), end='')
            print('')
        # 
        # import
        #print('globals()['+t_func_name+']')
        #globals()[t_func_name] = t_func_self
        #locals()[t_func_name] = t_func_self
        module = sys.modules[__name__]
        #print(module)
        if not hasattr(module, t_func_name):
            #print('setattr(module, '+t_func_name+', t_func_self)')
            setattr(module, t_func_name, t_func_self)





# 
# list all functions
# 
def help():
    print('Definitions:')
    print('    z        = galaxy\'s redshift')
    print('    SFR      = galaxy\'s star formation rate in units of solar mass per year')
    print('    Mstar    = galaxy\'s stellar mass in units of solar mass')
    print('    lgMstar  = log10(Mstar)')
    print('    sSFR     = SFR/Mstar*1e9 is a galaxy\' specific-SFR in units of Gyr^{-1}')
    print('    SFR_MS   = calc_SFR_MS(z, lgMstar) is a galaxy\'s main sequence SFR depending on its z and Mstar')
    print('    DeltaMS  = log10(SFR/SFR_MS) is a galaxy\'s offset to the galaxy main sequence')
    print('    deltaGas = Mmolgas/Mstar is a galaxy\'s molecular gas to stellar mass ratio')
    print('    muGas    = deltaGas')
    print('    fGas     = Mmolgas/(Mstar+Mmolgas) is a galaxy\'s molecular gas to total mass fraction')
    print('    tauGas   = Mmolgas/SFR*1e9 is a galaxy\'s molecular gas depletion time in units of Gyr')
    print('    tauDepl  = tauGas')
    print('    MetalZ   = 12+log10(O/H) is a galaxy\'s gas phase metallicity')
    print('    fmol     = Mmolgas/(Matomicgas+Mmolgas) is a galaxy\'s molecular fraction compared to total atomic+molecular gas')
    print('    GDR      = Mtotalgas/Mdust is a galaxy\'s total atomic+molecular gas to dust mass ratio')
    print('    deltaGDR = GDR')
    print('')
    print('Functions:')
    print('    # To calculate SFR with z and Mstar using galaxy main sequence correlations: ')
    load_all_functions_in_one_module(calc_galaxy_main_sequence, r'^calc_SFR_MS_.*', print_to_screen = True) # print all function within this module
    print('')
    print('    # To calculate gas fraction (deltaGas or muGas) with z, Mstar and SFR using gas scaling relations (note that here is muGas not fGas): ')
    load_all_functions_in_one_module(calc_gas_fraction, r'^calc_gas_fraction_.*', print_to_screen = True) # print all function within this module
    print('')
    print('    # To calculate gas depletion time (tauDepl or tauGas) with z, Mstar and SFR using gas scaling relations: ')
    load_all_functions_in_one_module(calc_gas_depletion_time, r'^calc_gas_depletion_time_.*', print_to_screen = True) # print all function within this module
    print('')
    
    print('Examples:')
    print('    import a3cosmos_gas_evolution as a3g')
    print('    a3g.calc_gas_fraction_A3COSMOS(z=3.0, lgMstar=10.5, DeltaMS=0.5)')
    print('    = %s'%(calc_gas_fraction_A3COSMOS(z=3.0, lgMstar=10.5, DeltaMS=0.5)))
    print('    a3g.calc_gas_depletion_time_A3COSMOS(z=3.0, lgMstar=10.5, DeltaMS=0.5)')
    print('    = %s'%(calc_gas_depletion_time_A3COSMOS(z=3.0, lgMstar=10.5, DeltaMS=0.5)))
    print('    # which means that a galaxy at redshift 3.0 with stellar mass 10^{10.5} solMass and 0.5 dex above the main sequence')
    print('    # has a molecular gas mass 3.84 times its stellar mass, and the molecular gas will be consumed possibly via ')
    print('    # star formation activities within only 0.32 billion years.')
    print('    ')
    
    
    #'calc_galaxy_main_sequence.py'
    #'calc_gas_depletion_time.py'
    #'calc_gas_fraction.py'
    #'calc_star_formation_law.py'
    #'calc_stellar_mass_function.py'
    #'calc_stellar_mass_function.py'
    #'calc_alpha_CO.py'
    #'calc_alpha_RJ.py'
    #'calc_cosmic_star_formation_rate_density.py'
    #'calc_delta_GD.py'
    #'calc_fmol.py'




# 
# main function
# 
if __name__ == '__main__':
    help()


